@@ -3,12 +3,17 @@ import os
 import traceback
 import pdb
 import bdb
+import warnings
 from copy import deepcopy
 from typing import TYPE_CHECKING, Union, List, Callable, Tuple, Optional, Any, Sequence, Type, cast, Dict
 
 from pyfileconf.basemodels.registrar import Registrar
 from pyfileconf.data.models.collection import SpecificClassCollection
+from pyfileconf.data.models.dictconfig import SpecificClassDictConfig
+from pyfileconf.dictfiles.modify import add_item_into_nested_dict_at_section_path, \
+    create_dict_assignment_str_from_nested_dict_with_ast_names, pretty_format_str
 from pyfileconf.pipelines.models.collection import PipelineCollection
+from pyfileconf.pipelines.models.dictconfig import PipelineDictConfig
 
 if TYPE_CHECKING:
     from pyfileconf.runner.models.interfaces import RunnerArgs, StrOrView
@@ -31,6 +36,7 @@ class PipelineManager:
     """
     Main class for managing flow-based programming and configuration.
     """
+    _active_managers: Dict[str, 'PipelineManager'] = {}
 
     def __init__(self, folder: str,
                  name: str= 'project',
@@ -55,8 +61,10 @@ class PipelineManager:
         self._general_registrar: Optional[PipelineRegistrar] = None
 
         self._validate_options()
+        self._active_managers[self.name] = self
         self._create_project_if_needed()
 
+
     def __getattr__(self, item):
 
         if item == 'runner':
@@ -227,6 +235,75 @@ class PipelineManager:
         """
         self.runner.update(d, section_path_str, **kwargs)
 
+    def create(self, section_path_str: str, func_or_class: Optional[Union[Callable, Type]] = None):
+        """
+        Create a new configuration entry dynamically rather than manually modifying dict file
+
+        :param func_or_class: function or class to use to generate config
+        :param section_path_str: section path at which the config should be stored
+        :return:
+        """
+        section_path = SectionPath(section_path_str)
+
+        if section_path[0] in self.specific_class_names:
+            # Got path for a specific class dict, add to specific class dict
+            if len(section_path) < 3:
+                raise ValueError('when targeting a specific class dict, section path must have minimum length of '
+                                 '3, e.g. example_class.thing.stuff')
+            if func_or_class is not None:
+                raise ValueError('only pass func_or_class when targeting main pipeline dict, not specific class dict')
+            self._create_specific_class_dict_entry(section_path)
+        else:
+            # Got path for main pipeline dict, add to main pipeline dict
+            if func_or_class is None:
+                raise ValueError('when adding creating item in main pipeline dict, must pass function or class')
+            self._create_pipeline_dict_entry(section_path, func_or_class)
+
+        # TODO [#64]: create items without reloading by inserting directly into registrar and config manager
+        self.reload()
+
+    def _create_specific_class_dict_entry(self, section_path: SectionPath):
+        name = section_path[0]
+        file_path = os.path.join(self.folder, f'{name}_dict.py')
+        specific_class_dict_file = SpecificClassDictFile(file_path, name=name + '_dict')
+        specific_dict = specific_class_dict_file.load()
+
+        # Modify specific class dict to add entry
+        add_item_into_nested_dict_at_section_path(
+            specific_dict,
+            # skip first section as that is name of specific class, skip last section as
+            # that's name which should be created
+            section_path[1:-1],
+            str(section_path[-1]),  # last section is name that should be created
+            add_as_ast_name=False
+        )
+        specific_dict_str = pretty_format_str('class_dict = ' + str(specific_dict))
+
+        specific_class_dict_config = SpecificClassDictConfig(
+            dict(class_dict=specific_dict_str),
+            name='specific_class_dict',
+            _file=specific_class_dict_file,
+        )
+        specific_class_dict_file.save(specific_class_dict_config)
+
+    def _create_pipeline_dict_entry(self, section_path: SectionPath, func_or_class: Union[Callable, Type]):
+        pipeline_dict_file = PipelineDictFile(self.pipeline_dict_path, name='pipeline_dict')
+        pipeline_dict = pipeline_dict_file.load()
+
+        # Modify pipeline dict to add entry
+        add_item_into_nested_dict_at_section_path(pipeline_dict, section_path, func_or_class.__name__)
+
+        # Convert pipeline dict to string for output
+        pipeline_dict_str = create_dict_assignment_str_from_nested_dict_with_ast_names(pipeline_dict)
+
+        pipeline_dict_config = PipelineDictConfig(
+            dict(pipeline_dict=pipeline_dict_str),
+            name='pipeline_dict',
+            _file=pipeline_dict_file,
+        )
+        pipeline_dict_file.save(pipeline_dict_config)
+
+
     def _load_pipeline_config_and_runner(self) -> None:
         """
         External logic for main initialization
@@ -272,6 +349,9 @@ class PipelineManager:
     def _validate_options(self):
         if self.auto_pdb and self.force_continue:
             raise ValueError('cannot force continue and drop into pdb at the same time')
+        if self.name in self._active_managers:
+            warnings.warn(f'should not repeat names of PipelineManagers. Got repeated name {self.name}. '
+                          f'The original PipelineManager has been replaced in the Selector.')
 
     def _create_project_if_needed(self):
         if self._need_to_create_project():
@@ -289,7 +369,10 @@ class PipelineManager:
             ],
         ])
 
-
+    def __del__(self):
+        current_manager_under_this_key = self._active_managers[self.name]
+        if current_manager_under_this_key is self:
+            del self._active_managers[self.name]
 
 
 def _try_except_run_func_except_user_interrupts(try_func: Callable, except_func: Callable = lambda x: x,
